-------------------------------------------------------------------------------
-- Title   : cPCI_statusManager
-- Project : TimEX3
-------------------------------------------------------------------------------
-- Description : Status register manager
--               The 2 last bits of the status register are dedicated to
--               undefined address read / write access
-------------------------------------------------------------------------------
-- File     : cPCI_statusManager.vhdl
-- Revision : x.x.x
-- Created  : December 05, 2012
-- Updated  : April 29, 2015
-------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
-------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2014 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
-------------------------------------------------------------------------------
--
-------------------------------------------------------------------------------
-- Modifications :
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.registers_init.all;

-------------------------------------------------------------------------------
--------------------------------- ENTITY --------------------------------------
-------------------------------------------------------------------------------
entity cPCI_statusManager is
  port (
    p_inStates : in  std_logic_vector (11 downto 0);
    p_status   : out std_logic_vector (29 downto 0);
    p_cfgSW    : in  std_logic_vector (5 downto 0);
    p_clk60MHz : in  std_logic;
    p_reset    : in  std_logic
  );
end entity cPCI_statusManager;

-------------------------------------------------------------------------------
------------------------------- ARCHITECTURE ----------------------------------
-------------------------------------------------------------------------------
architecture rtl_cPCI_statusManager of cPCI_statusManager is
  -----------------------------------------------------------------------------
  -- constant
  -----------------------------------------------------------------------------

  -----------------------------------------------------------------------------
  -- signal
  -----------------------------------------------------------------------------
  signal s_status : std_logic_vector (9 downto 0);

  -----------------------------------------------------------------------------
  -- Components
  -----------------------------------------------------------------------------

  -----------------------------------------------------------------------------
  --------------------------------- Main --------------------------------------
  -----------------------------------------------------------------------------
  begin

  process (p_reset, p_clk60MHz)
  begin
    if (p_reset = '1') then
      s_status <= c_status_reg_init;
    elsif (falling_edge(p_clk60MHz)) then
      case p_cfgSW is
        when "000000" => s_status <= "000000000" & p_inStates(0);
        when "000001" => s_status <= "000000"    & p_inStates(4 downto 1);
        when "000010" => s_status <= "000000000" & p_inStates(5);
        when "000011" => s_status <= "0000000"   & p_inStates(8 downto 6);
        when "000100" => s_status <= "000000000" & p_inStates(9);
        when "000101" => s_status <= "000000000" & p_inStates(10);
        when "000111" => s_status <= "000000000" & p_inStates(11);
        when others   => s_status <= "0000000000";
      end case;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------
  p_status <= "00000000000000000000" & s_status;

end architecture rtl_cPCI_statusManager;

