--------------------------------------------------------------------------------
-- Title   : Inhibition - LINAC monitoring function
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : If the inhibition is at low level, the outputs are inhibited
--
--------------------------------------------------------------------------------
-- File     : linacMON_inhibition.vhdl
-- Revision : x.x.x
-- Created  : November 20, 2014
-- Updated  : November 20, 2014
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2014 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.registers_init.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity linacMON_inhibition is
  port (
    p_reset      : in  std_logic;
    p_clk60MHz   : in  std_logic;
    p_inputs     : in  std_logic_vector (1 downto 0); -- input TTL
    p_inhibition : in  std_logic;
    p_outputs    : out std_logic_vector (1 downto 0)
  );
end entity linacMON_inhibition;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_linacMON_inhibition of linacMON_inhibition is

  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_holdInhibit : std_logic_vector (2 downto 0); -- synchronous inhibition

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- Synchronization of the external async inhibition signal with the 60MHz clock
  process (p_clk60MHz, p_reset)
  begin
    if (p_reset = '1') then
      s_holdInhibit <= (OTHERS => '0');
    elsif (rising_edge(p_clk60MHz)) then
      s_holdInhibit(0) <= p_inhibition;
      s_holdInhibit(1) <= s_holdInhibit(0);
      s_holdInhibit(2) <= s_holdInhibit(1);
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_outputs(0) <= p_inputs(0) and s_holdInhibit(2);
  p_outputs(1) <= p_inputs(1) and s_holdInhibit(2);

end architecture rtl_linacMON_inhibition;
