--------------------------------------------------------------------------------
-- Title   : Top - test function
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : Just blink LEDs for first of the board
--------------------------------------------------------------------------------
-- File     : test_top.vhdl
-- Revision : x.x.x
-- Created  : Febuary 22, 2013
-- Updated  : Febuary 22, 2013
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the Lesser GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity test_top is
  port (
    p_test_outTTL    : out std_logic_vector (4 downto 0); -- TTL outputs
    p_test_outPECL   : out std_logic_vector (4 downto 0); -- LVPECL outputs
    p_test_clk500mHz : in  std_logic;
    p_test_reset     : in  std_logic;
    p_test_led       : out std_logic_vector (1 downto 0)
  );
end entity test_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_test_top of test_top is

  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_led : std_logic_vector (1 downto 0);

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- Red LED
  -- The red LED is off after a reset. It gos ON if no input signal is detected
  -- after n mn.
  process (p_test_reset, p_test_clk500mHz)
  begin
    if (p_test_reset = '1') then
      s_led(0) <= '0'; -- red LED OFF
      s_led(1) <= '1'; -- green LED OFF
    elsif (rising_edge(p_test_clk500mHz)) then
      s_led(0) <= not s_led(0);
      s_led(1) <= not s_led(1);
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_test_led     <= s_led;
  p_test_outTTL  <= (OTHERS => '0');
  p_test_outPECL <= (OTHERS => '0');

end architecture rtl_test_top;
