--------------------------------------------------------------------------------
-- Title   : frequency division for DG slicing testbench
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : frequency division for DG slicing ; test bench
--------------------------------------------------------------------------------
-- File     : freqDiv_clkdiv_tb.vhd
-- Revision : x.x.x
-- Created  : October 30, 2012
-- Updated  : February 26, 2015
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity freqDiv_clkdiv_tb is
end entity freqDiv_clkdiv_tb;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture archi_freqDiv_clkdiv_tb of freqDiv_clkdiv_tb is

  -- constants
  constant Period_SR  : TIME := 1.181140 us;

  -- signals
  signal s_clkSR   : std_logic;
  signal s_rst     : std_logic;
  signal s_clkOUT  : std_logic_vector (4 downto 0);
  signal s_outPECL : std_logic_vector (4 downto 0);

  ----------------------------------------------------------------------------
  ----------------------------------------------------------------------------
  begin

  -- Frequency divider
  DUT : entity work.freqDIV_clkdiv (rtl_freqDIV_clkdiv)
  port map (
    p_clkSR   => s_clkSR,
    p_reset   => s_rst,
    p_clkOUT  => s_clkOUT,
    p_outPECL => s_outPECL
  );

  -- Clock generator
  process
  begin
    s_clkSR <= '0';
    wait for (Period_SR / 2);
    s_clkSR <= '1';
    wait for (Period_SR / 2);
  end process;

  process
  begin
    -- Init states
    s_rst <= '0';
    wait for 10 ns;
    s_rst <= '1';
    wait for 10 ns;
    s_rst <= '0';

    -- *** END ***
    wait;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------

end architecture archi_freqDiv_clkdiv_tb;
