--------------------------------------------------------------------------------
-- Title   : LINAC synchronized interlock test bench
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : LINAC synchronized interlock test bench
--------------------------------------------------------------------------------
-- File     : linacSYNCLOCK_td.vhdl
-- Revision : x.x.x
-- Created  : Decembre 02, 2014
-- Updated  : Decembre 02, 2014
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2014 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program. If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity linacSYNCLOCK_tb is
end entity linacSYNCLOCK_tb;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture simu_linacSYNCLOCK_tb of linacSYNCLOCK_tb is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------
  constant Period_60MHz : TIME := 16.67 ps;
  constant Period_3mHz  : TIME := 340 us;

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_reset      : std_logic;
  signal s_clk60MHz   : std_logic;
  signal s_clk3mHz    : std_logic;
  signal s_state      : std_logic;
  signal s_clkinTTL   : std_logic_vector (1 downto 0);
  signal s_clkoutTTL  : std_logic_vector (4 downto 0);
  signal s_clkoutPECL : std_logic_vector (4 downto 0);
  signal s_outLED     : std_logic_vector (1 downto 0);

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  DUT : entity work.linacSYNCLOCK_top (rtl_linacSYNCLOCK_top)
    port map (
      p_linacSYNCLOCK_inTTL    => s_clkinTTL,
      p_linacSYNCLOCK_outTTL   => s_clkoutTTL,
      p_linacSYNCLOCK_outPECL  => s_clkoutPECL,
      p_linacSYNCLOCK_clk60MHz => s_clk60MHz,
      p_linacSYNCLOCK_reset    => s_reset,
      p_linacSYNCLOCK_state    => s_state,
      p_linacSYNCLOCK_led      => s_outLED
    );

  -- Clock generator
  process
  begin
    s_clk60MHz <= '0';
    wait for (Period_60MHz / 2);
    s_clk60MHz <= '1';
    wait for (Period_60MHz / 2);
  end process;

  process
  begin
    s_clk3mHz <= '1';
    wait for (Period_3mHz / 2);
    s_clk3mHz <= '0';
    wait for (Period_3mHz / 2);
  end process;

  process
  begin
    -- Reset
    s_clkinTTL(1) <= '0';
    s_reset <= '0';
    wait for (Period_60MHz);
    s_reset <= '1';
    wait for (Period_60MHz);
    s_reset <= '0';

    ---------------------------
    ---------------------------
    wait for (19.15 us);
    s_clkinTTL(1) <= '1';
    wait for (10 us);
    s_clkinTTL(1) <= '0';
    wait for (310.85 us);

    ---------------------------
    ---------------------------
    wait for (15 us);
    s_clkinTTL(1) <= '1';
    wait for (4.15 us);
    s_clkinTTL(1) <= '0';
    wait for (290 us);


    ---------------------------

    -- *** END ***
    wait;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------
  s_clkinTTL(0) <= s_clk3mHz;

end architecture simu_linacSYNCLOCK_tb;
